/*
 * Copyright 2010-2011 Ning, Inc.
 *
 * Ning licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package com.ning.billing.util.queue;

import java.util.UUID;

import org.joda.time.DateTime;

public interface PersistentQueueEntryLifecycle {

    public enum PersistentQueueEntryLifecycleState {
        AVAILABLE,
        IN_PROCESSING,
        PROCESSED,
        REMOVED
    }

    public Long getTenantRecordId();

    public Long getAccountRecordId();

    public String getOwner();

    public String getCreatedOwner();

    public DateTime getNextAvailableDate();

    public PersistentQueueEntryLifecycleState getProcessingState();

    public boolean isAvailableForProcessing(DateTime now);

    // User token associated with this bus event or notification (i.e. user token from the context that
    // was used to generate this PersistentQueueEntryLifecycle)
    public UUID getUserToken();
}
